*************************************************************************
*                         NKC - Plotter by sEn                          *
*************************************************************************
*                                                                       *
* Realisierung eines Plotters mit 2 Schrittmotoren fuer die             *
* horizontale Achse und 1 Schrittmotor fr die Vertikale Achse          *
*                                                                       *
* - Ansteuerung der Schrittmotoren mit Steuergraet DM420 von ACT Motor  *
* - Ansteuerung Magnet fr Stift direkt mit BC548 und TIP120            *
* - Ansteuerung mit 1 I/O-Port (74LS245 - lesen, 74LS374 - schreiben)   *
*    1. Endlagenschalter horizontale / vertikale Achse =  $FFFFFFFF - R *
*    2. DM420 - horizontale / vertikale Achse und Stift = $FFFFFFFF - W *
*    3. Die Wartezeit zwischen 2 Motorpulsen ist ggfs, anzupassen!      *
*                                                                       *
*************************************************************************
*                                                                       *
*       Version 01 - 24.08.2021         Test Signale                    *
*       Version 02 - 20.09.2021         Plotterinitialisierung          *
*       Version 03 - 22.09.2021         Malen eines Quadrats            *
*       Version 04 - 01.10.2021         Manuelles Positionieren         *
*       Version 05 - 10.10.2022         I2C- auf Parallelansteuerung    *
*       Version 06 - 01.11.2021         Anpassung an Untersetzung 10:1  *
*       Version 07 - 01.12.2021         Malen eines Kreises             *
*       Version 08 - 01.04.2022         Fahren in Parkposition          *
*       Version 09 - 30.04.2022         Hard Copy Plot der GDP64HS      *
*       Version 10 - 19.05.2022         Erste Version                   *
*                                                                       *
*************************************************************************

;* Bei Assenblieren mit JADOS nicht verwenden!

ORG $50000


* * * *  Programmstart  * * * *

start:
bra plotinit


;*** Schicht 0 - Vereinbarungsteil Konstanten

;* NKC - Basis

; CPU

CPU             equ 1                   ; 68008

; GDP

GDP             equ $ffffff70*CPU       ;GDP-Basis
GDPHC           equ $ffffff60*CPU       ;DMA

;* Start des Bilschirmspeichers und Vergroeerung fuer Plotten einer Hardcopy

imgmem          equ $30000              ;16 KB werden bnoetigt
plotzoom        equ 500                 ;Zoom beim Plott einer Hardcopy

;* I/O-Port

; Endlagenschalter - lesen, Schrittmotoren - schreiben

port            equ $FFFFFFFF

;Bitkombinationen zur Motorsteuerung
;7=PEN, 6=DirectionH, 5 = PulseH, 4=DirV, 3=PV, 2=0, 1=EndlageV, 0 =EndlageH
;Standard = 1 Puls

;Bitkombination zum Lesen der Endlagenschalter
;7-4 = not set 3=vertikal oben, 2=vertikal unten, 1=horizontal links,
;0=horizontal rechts

VERTO1          equ %00001011           ;vertikal nach oben
VERTO0          equ %00000011
VERTOP1         equ %10001011           ;vertikal nach oben mit PEN
VERTOP0         equ %10000011
VERTO1E         equ %00001001           ;vertikal nach oben aus Endlage
VERTO0E         equ %00000001
VERTU1          equ %00011011           ;vertikal nach unten
VERTU0          equ %00010011
VERTUP1         equ %10011011           ;vertikal nach unten mit PEN
VERTUP0         equ %10010011
VERTU1E         equ %00011001           ;vertikal nach unten aus Endlage
VERTU0E         equ %00010001
HORR1           equ %00100011           ;horizontal nach rechts
HORR0           equ %00000011
HORRP1          equ %10100011           ;horizontal nach rechts mit PEN
HORRP0          equ %10000011
HORR1E          equ %00100010           ;horizontal nach rechts aus Endlage
HORR0E          equ %00000010
HORL1           equ %01100011           ;hortizontal nach links
HORL0           equ %01000011
HORLP1          equ %11100011           ;horizontal nach links mit PEN
HORLP0          equ %11000011
HORL1E          equ %01000010           ;horizontal nach links aus Endlage
HORL0E          equ %01100010
PENON           equ %10000011           ;K e i n  Schritt und PEN senken
PENOFF          equ %00000011           ;K e i n  Schritt und PEN heben

;* Fehlermeldungen und Statusmeldungen

err01:          dc.b 'xxx    Initialisation error - PEN in END position!',0
err02:          dc.b 'xxx    Horizontal / vertical endmark  n o t  found!',0
sta01:          dc.b '***************************************************',10
                dc.b '***         NKC-Plotter by sEn (c) 2021         ***',10
                dc.b '***************************************************',0
sta02:          dc.b '***    Checking horizontal / vertikal end marks ...',0
sta03:          dc.b '***    Horizontal / vertical end marks found!',10
                dc.b '***    Initial positioning of PEN ...',0
sta04:          dc.b '***    Plotter ready!',0

ds 0


; *** Schicht 1 - Routinen Variableninitialisierung, Fehler- /
;                 Statusausgabe, Warteroutine, Hardcopy Routine,
;                 Plotterinitialisierung, Plottergrundoperationen,
;                 Zeichnen eines Vektors, Testbild fuer Hardcopy,
;                 Anzeige Testbild aus Speicher


;* Initialisierung aller Variablen

varinit:
 lea varstart(pc), A0
 movea.l A0, A4
 move varanz(A0), d0
 subq #1, d0
 varclr:                         ;Aalle Bytes aller Variablen auf $00
 clr.b (A0)+
 dbra d0, varclr
 lea varstart(pc), A4
rts

;* Ausgabe von Fehlern und Status

status:
 move.b #$11, d0
 move #0, d1
 moveq #!writelf, d7
 trap #1
rts

;* Dynamisch fallende Wartezeit zwischen 2 Motorpulsen - ausgehend von zeit(a4)
;* Das ist die "Rampe" zum Anfahren eines Schritmotors!
;* Ohne gibt es Schrittverlust bei der verwendeten Endgescheindigkeit!

warte:
 cmp #0, zeit(a4)
 beq.s rampe
 sub #4, zeit(a4)
 rampe:
 move zeit(a4), d6
 pause:
  nop
 dbra d6, pause
rts

;* Initialisierung des Plotters - Check der Endlagenschalter und Positionierung
;* in die Mitte des Zeichenblatts

initplot:
 lea sta02(pc), a0              ;Endlagenschalter werden gesucht.
 move #160, d2
 bsr status

 move.b port, d5                ;Check, ob kein Endlagenschalter ausgelst ist
 cmp.b #%11111111, d5
 bne iniperr1                   ;Wenn ja, normale Initialisierung, sonst Fehler

 ;Anschlag vertikal oben suchen ...
 move #400, zeit(a4)            ;Wartezeit zwischen Motorpulsen
 verob:
 move #8-1, d0
 verob1:
  bsr vo                        ;1 Schritt vertikal nach oben = 8 Micro Steps
 dbra d0, verob1
 move.b port, d5
 btst.b #3, d5                  ;Endlagenschalter vertikal oben ausgeloest?
 bne.s verob

 ;Aus Endlage vertikal oben fahren ...
 move #400, zeit(a4)            ;Wartezeit zwischen Motorpulsen
 vereno:
 move #8-1, d0
 vereno1:
  bsr vu                        ;1 Schritt vertikal nach unten = 8 Micro Steps
 dbra d0, vereno1
 move.b port, d5
 btst.b #3, d5                  ;Endlagenschalter vertikal oben frei!
 beq.s vereno

 ;Anschlag vertikal unten suchen ...
 clr.l d1                       ;Initialisierung Schrittzaehler vertikal
 move #400, zeit(a4)            ;Wartezeit zwischen Motorpulsen
 vertunten:
  move #8-1, d0
  vertun1:
   bsr vu                       ;1 Schritt vertikal nach unten = 8 Micro Steps
  dbra d0, vertun1
  addq.l #1, d1                 ;Schritte vertikal zaehlen ...
  move.b port, d5
  btst.b #2, d5                 ;Endlagenschalter vertikal unten ausgeloest?
  bne.s vertunten
  move.l d1, scaley(a4)         ;Gesamtzahl Schritte Y-Achse

 ;Aus Endlage vertikal unten fahren ...
 move #400, zeit(a4)            ;Wartezeit zwischen Motorpulsen
 vertendu:
  move #8-1, d0
  vendu1:
   bsr vo                       ;1 Schritt vertikal nach oben/8 Micro Steps
  dbra d0, vendu1
  move.b port, d5
  btst.b #2, d5                 ;Endlagenschalter vertikal unten frei
  beq.s vertendu

 ;Anschlag horizontal rechts suchen ...
 move #400, zeit(a4)            ;Wartezeit zwischen Motorpulsen
 horechts:
  move #8-1, d0
  hore1:
   bsr hr                       ;1 Schritt horizontal nach rechts = 8 Micro Step
  dbra d0, hore1
  move.b port, d5
  btst.b #0, d5                 ;Endlagenschalter horizontal rechts ausgeloest?
  bne.s horechts

 ;Aus Endlage horizontal rechts fahren ...
 move #400, zeit(a4)            ;Zeit zwischen 2 Motorpulsen
 horendr:
 move #8-1, d0
 hendr1:
  bsr hl                        ;1 Schritt horizontal nach links = 8 Micro Steps
 dbra d0, hendr1
 move.b port, d5
 btst.b #0, d5                  ;Endlagenschalter horizontal rechts frei
 beq.s horendr

 ;Anschlag horizontal links suchen ...
 clr.l d1                       ;Initialisierung Schrittzaehler horizontal
 move #400, zeit(a4)
 horlinks:
  move #8-1, d0
  holi1:
   bsr hl                       ;1 Schritt horizontal nach links = 8 Micro Steps
  dbra d0, holi1
  addq.l #1, d1                 ;Schritte horizontal zaehlen ...
  move.b port, d5
  btst.b #1, d5                 ;Endlagenschalter horizontal links ausgeloest?
  bne.s horlinks
  move.l d1, scalex(a4)         ;Gesamtanzahl Schritte X-Achse

 ;Aus Endlage horizontal links fahren ...
 move #400, zeit(a4)            ;Zeit zwischen 2 Motorpulsen
 horendl:
  move #8-1, d0
  hendl1:
   bsr hr                       ;1 Schritt horizontal nach rechts = 8 Micro Step
  dbra d0, hendl1
  move.b port, d5
  btst.b #1, d5                 ;Endlagenschalter horizontal links frei?
  beq.s horendl

 lea sta03(pc), a0              ;Endlagenschalter gefunden.
 move #150, d2
 bsr status

 ;PEN in die Mitte des Zeichenblatts fahren
 move.l scaley(a4), d1          ;Halbe Anzahl Gesamtschritte Y-Achse
 divu #2, d1
 subq.l #1, d1
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 midy:
  move #8-1, d0
  midy1:
   bsr vo                       ;1 Schritt vertikal nach oben = 8 Micro Steps
  dbra d0, midy1
 dbra d1, midy
 move.l scalex(a4), d1          ;Halbe Abzahl Gsamtschritte X-Achse
 divu #2, d1
 subq.l #1, d1
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 midx:
  move #8-1, d0
  midx1:
   bsr hr                       ;1 Schritt horizontal nach rechts = 8 Micro Step
   bsr warte
  dbra d0, midx1
 dbra d1, midx

 lea sta04(pc), a0              ;Plotter ready!
 move #130, d2
 bsr status

 bra.s inipend

 iniperr1:                      ;PEN in einer Endlage - nicht erlaubt bei
 lea err01(pc), a0              ;Initialisierung!
 move #150, d2
 bsr status
 bra.s inipend

iniperr2:                       ;Endlagenschalter nicht erkannt!
lea err02(pc), a0
move #150, d2
bsr status

inipend:
rts

;* Grundlegende Plotteroperationen - Motor-Byte an port

;Vertikal nach oben mit PEN - Puls

vop:
 move.b #VERTOP1, port
 bsr warte
 move.b #VERTOP0, port
 bsr warte
rts

;Vertikal nach oben ohne PEN - Puls

vo:
 move.b #VERTO1, port
 bsr warte
 move.b #VERTO0, port
 bsr warte
rts

;Vertikal nach oben ohne PEN aus Endlage - Puls

voe:
 move.b #VERTO1E, port
 bsr warte
 move.b #VERTO0E, port
 bsr warte
rts

;Horizontal nach rechts mit PEN - Puls

hrp:
 move.b #HORRP1, port
 bsr warte
 move.b #HORRP0, port
 bsr warte
rts

;Horizontal nach rechts ohne PEN - Puls

hr:
 move.b #HORR1, port
 bsr warte
 move.b #HORR0, port
 bsr warte
rts

;Horizontal nach rechts ohne PEN aus Endlage - Puls

hre:
 move.b #HORR1E, port
 bsr warte
 move.b #HORR0E, port
 bsr warte
rts

;Vertikal nach unten mit PEN - Puls

vup:
 move.b #VERTUP1, port
 bsr warte
 move.b #VERTUP0, port
 bsr warte
rts

;Vertikal nach unten ohne PEN - Puls

vu:
 move.b #VERTU1, port
 bsr warte
 move.b #VERTU0, port
 bsr warte
rts

;Vertikal nach unten ohne PEN aus Endlage - Puls

vue:
 move.b #VERTU1E, port
 bsr warte
 move.b #VERTU0E, port
 bsr warte
rts

;Horizontal links mit PEN - Puls

hlp:
 move.b #HORLP1, port
 bsr warte
 move.b #HORLP0, port
 bsr warte
rts

;Horizontal links ohne PEN - Puls

hl:
 move.b #HORL1, port
 bsr warte
 move.b #HORL0, port
 bsr warte
rts

;Horizontal links ohne PEN aus Endlage - Puls

hle:
 move.b #HORL1E, port
 bsr warte
 move.b #HORL0E, port
 bsr warte
rts

;* Vektor mit bestimmter Laenge und bestimmtem Winkel zeichnen
;* Winkel in d0, Laenge in d1
;* Sinus- und Cosinuswerte von -256 bis +256 !

vektor:
 clr.l d4                       ;X-Komponente
 clr.l d5                       ;Y-Komponente

 ;Berechnung X-/Y-Komponente des Winkels
 movem.w d0, -(A7)              ;Winkel fuer Berechnung Y-Komponente retten
 moveq #!sin, d7                ;Berechnung X-Komponente
 trap #1
 move.w d0, d4                  ;X-Komponente Winkel in d4
 movem.w (A7)+, d0
 movem.w d0, -(A7)              ;D0 - Winkel retten fuer mehrfache Aufrufe
 moveq #!cos, d7                ;Berechnugn Y-Komponente
 trap #1
 move.w d0, d5                  ;Y-Komponente Winkel in d5

;Richtung X, Y bestimmen und in vzx, vzy ablegen
move.b #1, vzx(a4)              ;Vorzeichen X
move.b #1, vzy(a4)              ;Vorzeichen Y
clr.w faktor(a4)                ;Faktor fuer Komponente aus Verhaeltnis

btst #15, d4
beq.s xhpos
xhng:
move.b #-1, vzx(a4)
neg.w d4                        ;Asolutwert in d4
xhpos:
btst #15, d5
beq.s xvpos
xvneg:
move.b #-1, vzy(a4)
neg.w d5                        ;Absolutwert in d5
xvpos:

;Berechnung X-/Y-Komponente des Vektors
mulu d1, d4                     ;X-Komponente absolut des Vektors in d4
mulu d1, d5                     ;Y-Komponente absolut des Vektors in d5
move.l d4, xwert(a4)            ;X-Komponente retten
move.l d5, ywert(a4)            ;Y-Komponente retten
clr.w faktor(a4)                ;fuer Verhaeltnisberechnung

;Sonderfaelle senkrechter und waagrechter Vektor
;senkrechter Vektor
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 vert:
  cmp.l #0, d4
  bne.s waag
  cmp.b #-1, vzy(a4)            ;Richtung
  bne.s vpos
  vneg:
   subq.l #1, d5
   vneg1:
    bsr vup
   dbra d5, vneg1
  bra vekend
  vpos:
   subq.l #1, d5
   vpos1:
    bsr vop
   dbra d5, vpos1
  bra vekend
 ;waagrechter Vektor
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 waag:
  cmp.l #0, d5
  bne.s standard
  cmp.b #-1, vzx(a4)            ;Richtung
  bne.s hpos
  hneg:
   subq.l #1, d4
   hneg1:
    bsr hlp
   dbra d4, hneg1
  bra vekend
  hpos:
   subq.l #1, d4
   hpos1:
    bsr hrp
   dbra d4, hpos1
  bra vekend

;Standarfaelle
standard:
 ;Restwertspeicher
 clr.w xrest
 clr.w yrest
 ;Die groeere der beiden Komponenten X, Y bestimmen
 cmp.l d4, d5
 ble xloop

yloop:                          ;Schleife ueber Y-Komponente
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 move.l d5, d0
 subq.l #1, d0
 yloop1:
  move.l xwert(a4), d4
  move.l ywert(a4), d5
  add.w #1000, faktor(a4)
  mulu faktor(a4), d4           ;Berechnete X-Komponente aus Verhaeltnis
  divu d5, d4                   ;X/Y
  add.w xrest, d4               ;X-Restwert addieren
  yxplot:                       ;Y-Schritt H und X-Schritt H
   cmp.b #-1, vzy(a4)           ;Vertikale Richtung
   beq.s yxplotn                ;nach unten
   yxplotp:                     ;nach oben
   cmp.b #-1, vzx(a4)           ;Horizontale Richtung
   beq.s yxplotpn               ;nach links
   yxplotpp:
   bsr vop                      ;nach oben
   cmp.w #1000, d4              ;X-Schritt da?
   blt.s yplote                 ;Nur Y-Schritt
   bsr hrp                      ;X-Schritt rechts
   sub.w #1000,d4
   move.w d4, xrest             ;Restwert
   clr.w faktor(a4)             ;RESET X-Schritt
   bra.s yplote
   yxplotpn:
   bsr vop                      ;nach oben
   cmp.w #1000, d4              ;X-Schritt da?
   blt.s yplote                 ;Nur Y-Schritt
   bsr hlp                      ;X-Schritt links
   sub.w #1000,d4
   move.w d4, xrest             ;Restwert
   clr.w faktor(a4)             ;RESET X-Schritt
   bra.s yplote
   yxplotn:                     ;nach unten
   cmp.b #-1, vzx(a4)           ;Horizontale Richtung
   beq.s yxplotnn
   yxplotnp:
   bsr vup                      ;nach unten
   cmp.w #1000, d4              ;X-Schritt da?
   blt.s yplote                 ;Nur Y-Schritt
   bsr hrp                      ;X-Schritt rechts
   sub.w #1000,d4
   move.w d4, xrest             ;Restwert
   clr.w faktor(a4)             ;RESET X-Schritt
   bra.s yplote
   yxplotnn:
   bsr vup                      ;nach unten
   cmp.w #1000, d4              ;X-Schritt da?
   blt.s yplote                 ;Nur Y-Schritt
   bsr hlp                      ;X-Schritt links
   sub.w #1000,d4
   move.w d4, xrest             ;Restwert
   clr.w faktor(a4)             ;RESET X-Schritt
 yplote:
 dbra d0, yloop1
 bra vekend

xloop:                          ;Schleife ueber X-Komponente
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 move.l d4, d0
 sub.l #1, d0
 xloop1:
  move.l xwert(a4), d4
  move.l ywert(a4), d5
  add.w #1000, faktor(a4)
  mulu faktor(a4), d5           ;Berechnete Y-Komponente aus Verhaeltnis
  divu d4, d5                   ;Y/X
  add.w yrest, d5               ;Y-Restwert addieren
  xyplot:                       ;X-Schritt H und Y-Schritt H
   cmp.b #-1, vzx(a4)           ;Horizontale Richtung
   beq.s xyplotn
   xyplotp:                     ;nach rechts
   cmp.b #-1, vzy(a4)           ;Vertikale Richtung
   beq.s xyplotpn
   xyplotpp:
   bsr hrp                      ;nach rechts
   cmp.w #1000, d5              ;Y-Schritt da?
   blt.s xplote                 ;Nur X-Schritt
   bsr vop                      ;Y-Schritt oben
   sub.w #1000, d5
   move.w d5, yrest
   clr.w faktor(a4)             ;RESET Y-Schritt
   bra.s xplote
   xyplotpn:
   bsr hrp                      ;nach rechts
   cmp.w #1000, d5              ;Y-Schritt da?
   blt.s xplote                 ;Nur X-Schritt
   bsr vup                      ;Y-Schritt unten
   sub.w #1000, d5
   move.w d5, yrest
   clr.w faktor(a4)             ;RESET Y-Schritt
   bra.s xplote
   xyplotn:                     ;nach links
   cmp.b #-1, vzy(a4)           ;Vertikale Richtung
   beq.s xyplotnn
   xyplotnp:
   bsr hlp                      ;nach links
   cmp.w #1000, d5              ;Y-Schritt da?
   blt.s xplote                 ;Nur X-Schritt
   bsr vop                      ;Y-Schritt oben
   sub.w #1000, d5
   move.w d5, yrest
   clr.w faktor(a4)             ;RESET Y-Schritt
   bra.s xplote
   xyplotnn:
   bsr hlp                      ;nach links
   cmp.w #1000, d5              ;Y-Schritt da?
   blt.s xplote                 ;Nur X-Schritt
   bsr vup                      ;Y-Schritt unten
   sub.w #1000, d5
   move.w d5, yrest
   clr.w faktor(a4)             ;RESET Y-Schritt
  xplote:
 dbra d0, xloop1

vekend:
 movem.w (A7)+, d0              ;Alten Winkel zurueck geben
rts

;* Harcopy des Bilschirms erzeugen und im Speicher ablegen
;* Bildschirmspeicher der GDP64HS wird mit MFREE-Signal
;* auf Port $FFFFFF60 ausgelesen und ins RAM geschrieben
* X, Y - Register EF9366 (lesen und schreiben) - Adresse fuer 68008
*                   Y = Zeilen (256)   X = Bilpunkte pro Zeile (512)
* 7 6 5 4 3 2 1 0 - LSB = $FFFFFF7B    $FFFFFF79   0-255
* X X X X 3 2 1 0 - MSB = $FFFFFF7A    $FFFFFF78   256-511 (Bit 0 = 1)
*                         not used

hardcopy:
 lea GDPHC, A1                  ;GDP-DMA
 lea GDP, A2                    ;GDP-Basis
 movea.l A2, A3
 move.w #CPU, d0
 muls #$b, d0
 adda.l d0, A3                  ;LSB Y
 lea imgmem, A0                 ;Start Bildpeicher
 move #256-1, d2                ;Zeilen
 nzeile:
  btst.b #2, (A2)               ;Statusregister
  beq.s nzeile                  ;GDP beschaeftigt
  move.b d2, (A3)               ;Zeile in LSB Y-Register auswaehlen
  suba.l #CPU, A3               ;LSB X berechnen
  suba.l #CPU, A3
  clr d1                        ;Byte-Zaehler fr Bildpunkt
  moveq #64-1, d3               ;Byte-Zaehler Punkte - 64 * 8 = 512
  nbyte:
   btst #2, (A2)                ;Statusregister
   beq.s nbyte                  ;GDP beschaeftigt
   move.b d1, (A3)              ;Punkt in LSB X-Register in Zeile auswaehlen
   suba.l #CPU, A3              ;MSB X berechnen
   ror.w #8, d1                 ;MSB in d1 nach LSB schieben
   move.b d1, (A3)              ;Punkt in MSB X-Register in Zeile auswaehlen
   rol.w #8, d1                 ;d1 wieder korrekt schieben
   adda.l #CPU, A3              ;LSB wieder setzen
   move.b #$0F, (A2)            ;GDP-DMA-Befehl, Speicher auslesen
   readout:
    btst #2, (A2)               ;Statusregister
    beq.s readout               ;GDP beschaeftigt
    move.b (A1), d4             ;Register 74LS374 auf GDP auslesen
    move.b d4, (A0)+            ;Bildpunkti bilspeicher ablegen
    addq #8, d1                 ;Naechster Bildpunkt, naechstes Byte in Zeile
  dbra d3, nbyte                ;Bis alle 64 Bytes - 64 * 8 = 512 Bildpunkte
  adda.l #CPU, A3               ;LSB Y wieder setzen - Zeile
  adda.l #CPU, A3
 dbra d2, nzeile                ;Bis alle 256 Zeilen
rts

;* Testbild fuer Hardcopy - Vielquadrat

testbild:
 move #36-1, d4                 ;Aeussere Schleife - 36 Quadrate zeichnen
 aussen:
  move #4-1, d3                 ;Innere Schleife - Quadrat zeichnen
  innen:
   move #170, d0
   moveq #!schreite, d7
   trap #1
   move #90, d0
   moveq #!drehe, d7
   trap #1
  dbra d3, innen
  move #10, d0
  moveq #!drehe, d7
  trap #1
 dbra d4, aussen
rts

;* Testbild aus Speicher anzeigen

dispimg:
 lea imgmem, A0
 move #256-1, d2
 azeilen:
  clr d1
  move #64-1, d4
  apunkte:
   move.b (A0)+, d6
   move #8-1, d3
    pbyte:
     moveq #!moveto, d7
     trap #1
     btst.b #7, d6
     bne.s nopix
     moveq #!wait, d7
     trap #1
     moveq #$80, d0
     moveq #!cmd, d7
     trap #1
    nopix:
    rol.b #1, d6
    addq #1, d1                 ;naechster Punkt
    dbra d3, pbyte
  dbra d4, apunkte
 dbra d2, azeilen
 rts


; *** Schicht 2 - Hauptprogramme

;* Initialisierung des Plotters

plotinit:
 bsr varinit            ;Variableninitialisieung
 lea sta01(pc), A0      ;Bildschirminfo
 move #200, d2
 bsr status
 bsr initplot           ;Plotter initialisieren
rts

;* Manuelles Positionieren mit Tasten: o-ben, u-nten, r-echts, l-inks
;* Pen down = s, PEN up = h
;* Automatisches Fahren aus Endlage bei Programmstart und manuell mit Taste e

manpos:
 bsr varinit                    ;Variableninitialisierung
 clr.b d4                       ;Fuer Senken und Heben des PEN

;Fahren aus einer Endposition
 move.b port, d5                ;Endlagenschalter auslesen
 btst.b #3, d5                  ;Endlagenschalter vertikal oben ausgeloest?
 bne.s evu
 ;Aus Endlage vertikal oben fahren ...
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 swvo:
  bsr vue                       ;1 Schritt vertikal nach unten aus Endlage
  move.b port, d5
  btst.b #3, d5                 ;Endlagenschalter vertikal oben frei?
 beq.s swvo

 evu:
  btst.b #2, d5                 ;Endlagenschalter vertikal unten ausgeloest?
  bne.s ehr
  ;Aus Endlage vertikal unten fahren ...
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  swvu:
   bsr voe                      ;1 Schritt vertikal nach oben aus Endlage
   move.b port, d5
   btst.b #2, d5                ;Endlagenschalter vertikal unten frei?
   beq.s swvu

 ehr:
  btst #0, d5                   ;Endlagenschalter horizontal rechts ausgeloest?
  bne.s ehl
  ;Aus Endlage horizontal rechts fahren ...
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  swhr:
   bsr hle                      ;1 Schritt horizontal nach links
   move.b port, d5
   btst #0, d5                  ;Endlagenschalter horizontal rechts frei
  beq.s swhr

 ehl:
  btst.b #1, d5                 ;Endlagenschalter horizontal links ausgeloest?
  bne.s man
  ;Aus Endlage horizontal links fahren ...
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  swhl:
   bsr hre                      ;1 Schritt horizontal nach rechts
   move.b port, d5
   btst.b #1, d5                ;Endlagenschalter horizontal links frei?
  beq.s swhl

 ;Manuelles Positionieren

 man:
  moveq #!csts, d7
  trap #1
  beq.s man
  moveq #!ci, d7
  trap #1
  oben:                         ;Strecke nach oben fahren
  cmp.b #'o', d0
  bne.s unten
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  move #10000-1, d3             ;Feste Fshrstrecke
  oben1:
   move.b #VERTO1, d5
   add.b d4, d5
   move.b d5, port
   bsr warte
   move.b #VERTO0, d5
   add.b d4, d5
   move.b d5, port
   bsr warte
  dbra d3, oben1
  bra.s man
 unten:                         ;Strecke nach unten fahren
  cmp.b #'u', d0
  bne.s rechts
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  move #10000-1, d3             ;Feste Fahrtstrecke
  unten1:
   move.b #VERTU1, d5
   add.b d4, d5
   move.b d5, port
   bsr warte
   move.b #VERTU0, d5
   add.b d4, d5
   move.b d5, port
   bsr warte
  dbra d3, unten1
  bra.s man
  rechts:                       ;Strecke nach rechts fahren
   cmp.b #'r', d0
   bne.s links
   move #400, zeit(a4)          ;Zeit zwischen motorpulsen
   move #10000-1, d3            ;Feste Fahrtstrecke
   rechts1:
    move.b #HORR1, d5
    add.b d4, d5
    move.b d5, port
    bsr warte
    move.b #HORR0, d5
    add.b d4, d5
    move.b d5, port
    bsr warte
   dbra d3, rechts1
   bra man
 links:                         ;Strecke nach links fahren
  cmp.b #'l', d0
  bne.s pendown
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  move #10000-1, d3             ;Feste Fahrtstrecke
  links1:
   move.b #HORL1, d5
   add.b d4, d5
   move.b d5, port
   bsr warte
   move.b #HORL0, d5
   add.b d4, d5
   move.b d5, port
   bsr warte
  dbra d3, links1
  bra man
 pendown:                       ;Stift senken - mit jeder Operation kombiniert!
  cmp.b #'s', d0
  bne.s penup
  move #%10000000, d4
  move.b d4, port
  bra man
 penup:                         ;Stift heben - mit jeder Operation kombiniert
  cmp.b #'h', d0
  bne.s endlage
  clr.b d4
  move.b d4, port
  bra man
 endlage:                       ;Fahren aus einer Endlage
  cmp.b #'e', d0
  bne.s ende
  bra manpos
 ende:
  cmp.b #27, d0                  ;ESC
  bne man
rts

;* Kreis mit bestimmtem Radius zeichnen
;* Radius in D1
;* Sinus- und Cosinuswerte von -256 bis +256 !

circle:
 bsr varinit                    ;Initialisierung Variablen

 move.w #250, d1                ;Radius
 movem.w d1, -(A7)              ;Radius retten
 clr.l d4                       ;X-Komponente
 clr.l d5                       ;Y-Komponente

;* Einmal Radius senkrecht, um auf Kreislinie zu positionieren
 move.w #400, zeit(a4)          ;Wartezeit zwischen 2 Motorpulsen
 mulu #256, d1                  ;Skalierung, weil sin(0) = 256 !
 move.l d1, ywert(a4)           ;Erster Punkt / Bezug fuer Delta-Berechnung
 clr.l xwert(a4)
 subq.l #1, d1
 iradius:
  bsr vo                        ;vertikal oben ohne PEN
 dbra d1, iradius
 movem.w (A7)+, d1              ;Radius wieder herstellen

;* Jetzt alle Punkte der Kreislinie relativ zum zuvor gezeichneten Punkt malen

move.w #360-1, d2
pixel:
 addq.w #1, d2                  ;Damit von 360 bis 1

 ;Bestimmung des Kreissektors
 lo:                            ;links oben
  cmp.w #270, d2
  blt.s lu
  move.b #-1, vzx(a4)
  move.b #-1, vzy(a4)
  bra.s calcu
 lu:                            ;links unten
  cmp.w #180, d2
  blt.s ru
  move.b #1, vzx(a4)
  move.b #-1, vzy(a4)
  bra.s calcu
 ru:
  cmp.w #90, d2
  blt.s ro
  move.b #1, vzx(a4)
  move.b #1, vzy(a4)
  bra.s calcu
 ro:                            ;rechts oben
  move.b #-1, vzx(a4)
  move.b #1, vzy(a4)

;Berechnung X-/Y-Komponente des Winkels
calcu:
 move.w d2, d0
 moveq #!sin, d7                ;Berechnung X-Komponente
 trap #1
 cmp.w #0, d0
 bge.s xpos
 neg.w d0
 xpos:
  move.w d0, d4                 ;X-Komponente Winkel absolut in d4
  move.w d2, d0
  moveq #!cos, d7               ;Berechnung Y-Komponente
  trap #1
  cmp.w #0, d0
  bge.s ypos
  neg.w d0
 ypos:
  move.w d0, d5                 ;Y-Komponente Winkel absolut in d5

 ;Berechnung aktuelle X-/Y-Komponente des Punktes auf der Kreislinie
 mulu d1, d4                    ;X-Komponente absolut des Punktes in d4
 movem.l d4, -(A7)
 mulu d1, d5                    ;Y-Komponente absolut des Punktes in d5
 movem.l d5, -(A7)

 ;Differenzwerte zu vorhergehendem Punkt berechnen
 sub.l xwert(a4), d4            ;Delta-X in d4
 cmp.l #0, d4
 bge.s xppos
 neg.l d4
 xppos:
  sub.l ywert(a4), d5           ;Delta-Y in d5
  cmp.l #0, d5
  bge.s yppos
  neg.l d5
 yppos:
  move.l (A7)+ ,ywert(a4)       ;Vergleichswert fuer naechsten Punkt
  move.l (A7)+, xwert(a4)       ;Vergleichswert fuer naechsten Punkt

 ;Punkt malen
 cmp.w #0, d5                   ;Delta Y = 0 ? Ja, dann weiter mit Delta X.
 beq.s cxpix
 cypix:
  move.w #400, zeit(a4)         ;Zeit zwischen 2 Motorpulsen
  cmp.b #1, vzy(a4)
  beq.s ypp
  move.w d5, d3
  subq.w #1, d3
  ypn1:
   bsr vup
  dbra d3, ypn1
  bra.s cxpix
  ypp:
   move.w d5, d3
   subq.w #1, d3
   ypp1:
    bsr vop
   dbra d3, ypp1

 cxpix:
  cmp.w #0, d4                  ;Delta X = 0 ? Ja, dann wird kein Punkt gemalt.
  beq.s cpixe
  move #400, zeit(a4)           ;Zeit zwischen 2 Motorpulsen
  cmp.b #1, vzx(a4)
  beq.s xpp
  move.w d4, d3
  subq.w #1, d3
  xpn1:
   bsr hlp
  dbra d3, xpn1
  bra.s cpixe
  xpp:
   move.w d4, d3
   subq.w #1, d3
   xpp1:
    bsr hrp
   dbra d3, xpp1

 cpixe:
  subq.w #1, d2                 ;Korrektur fuer Schleife
 dbra d2, pixel
 move.b #PENOFF, port           ;Stift heben
rts

;* Malen eines Strichs mit Winkel und Laenge

line:
 bsr varinit                    ;Initialisierung Variablen
 move.w #280, d0                ;Winkel
 move.w #255, d1                ;Laenge
 bsr vektor                     ;Unterprogramm 'Vektor'
 move.b #PENOFF, port           ;Stift heben
rts

;* Plotten eines Vielquadrats mit Kantenlaenge in D1

vqplot:
 bsr varinit                    ;Initialisierung Variablen
 move.w #256, d1                ;Kantenlaenge
 move.w #0, d0                  ;Winkel
 move #36-1, d2                 ;Aeussere Schleife in 10 Grad Schritten
 schl2:
  move #4-1, d3                 ;Innere Schleife fr ein Quadrat
  schl1:
   bsr vektor                   ;Unterprogramm 'Vektor'
   add.w #90, d0
  dbra d3, schl1
  add.w #10, d0
  sub.w #360, d0
  weiter:
 dbra d2, schl2
 move.b #PENOFF, Port           ;Stift heben
rts

;* Malen eines Testbilds und Ablage der Hardcopy im Speicher

hacomem:
 bsr varinit                    ;Variableninitialisierung
 bsr testbild                   ;Testbild malen
 bsr hardcopy                   ;Aus GDP64HS in Speicher
 moveq #!clr, d7
 trap #1
 bsr dispimg
rts

;* Plot eines GDP64-Bildes aus dem Speicher
;* Das Bild wurde vorher als Hardcopy im Speicher abgelegt
;* 16K - 256 Zeilen mit jeweils 512 Bildpunkten

hacoplot:
 bsr varinit                    ;Variablenintialisierung

 ;Stift in Startposition fahren

 ;In Endlage vertikal oben fahren ...
 move #400, zeit(a4)            ;Zeit zwischen motorpulsen
 vertoben:
 move #8-1, d0
 vertob:
  bsr vo                        ;1 Schritt vertikal nach oben
 dbra d0, vertob
 move.b port, d5
 btst.b #3, d5                  ;Endlagenschalter vertikal oben ausgeloest?
 bne.s vertoben

 ;Aus Endlage vertikal oben fahren ...
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 verendo:
 move #8-1, d0
 vendo1:
  bsr vue                       ;1 Schritt vertikal nach unten aus Endlage
 dbra d0, vendo1
 move.b port, d5
 btst.b #3, d5                  ;Endlagenschalter vertikal oben frei?
 beq.s verendo

 ;In Endlage horizontal rechts fahren ...
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  rechts3:
   bsr hr                       ;1 Schritt horizontal nach rechts = 8 Micro Step
   move.b port, d5
   btst #0, d5                  ;Endlagenschalter horizontal rechts ausgeloest?
   bne.s rechts3

 ;Aus Endlage horizontal rechts fahren ...
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  swhr2:
   bsr hle                      ;1 Schritt horizontal nach links aus Endlage
   move.b port, d5
   btst #0, d5                  ;Endlagenschalter horizontal rechts frei?
  beq.s swhr2

 ;In die Startposition fahren
  clr.l d0
  move #400, zeit(A4)           ;Zeit zwischen motorpulsen
  oben3:
   bsr hl                       ;1 Schritt horizontal nach links
   addq.l #1, d0
   cmp.l #30000, d0
  bne.s oben3

 lea imgmem, A1                 ;Startadresse Bildspeicher

 move.w #128,d1                 ;Schleife - 256 Zeilen = 128 mal hin / zurueck
 zeilen:
  hin:                          ;Ploter schreibt von oben nach unten
   move #400, zeit(A4)          ;Rampe ungerade Zeile
   move #100, d4
   rampeu:
    bsr vu                      ;1 Schritt vertikal nach unten
   dbra d4, rampeu
   move #64-1, d3               ;64 Bytes = 512 Bits / Bildpunkte
   byteu:
    move.b (A1)+, d0            ;8 Bildpunkte
    move.w #8-1, d4
    plotu:
     btst.b #7, d0              ;Bildpunkt hell oder dunkel?
     bne.s dunkelu              ;Bildpunkt ist dunkel
     hellu:                     ;Bildpunkt ist hell - Stift unten
     move.w #plotzoom-1, d5     ;Punkt in zoom-facher Laenge
     zoomu1:
      bsr vup                   ;1 Schritt vertikal nach unten mit PEN
     dbra d5, zoomu1
     bra.s weiteru
     dunkelu:                   ;Bildpunkt ist dunkel - Stift oben
      move #plotzoom-1, d5      ;Punkt in zoom-facher Laenge
      zoomu2:
       bsr vu                   ;1 Schritt vertikal nach unten
      dbra d5, zoomu2
     weiteru:
      rol.b #1, d0              ;naechster Bildpunkt
    dbra d4, plotu
   dbra d3, byteu
   move #400, zeit(A4)          ;Vorbereitung Rampe gerade Zeile
   move.w #100, d4
   rampeu1:
    bsr vu                      ;1 Schritt vertikal nach unten
   dbra d4, rampeu1
  zendeh:
  move #400, zeit(A4)           ;Zeilenabstand
  move.w #plotzoom, d4
  mulu #2, d4                   ;Es gibt nur 256 Zeilen aber 512 Bildpunkte!
  zah:
   bsr hl                       ;1 Schritt horizontal nach links
  dbra d4, zah

  zurueck:                      ;Plotter schreibt von unten nach oben
  adda #64, A1                  ;Zeile von hinten lesen
  move #400, zeit(A4)           ;Rampe gerade Zeile
  move.w #100, d4
  rampeo:
   bsr vo                       ;1 Schritt vertikal nach oben
  dbra d4, rampeo
  move #64-1, d3                ;64 Bytes = 512 Bits / Bildpunkte
  byteo:
   move.b -(A1), d0             ;8 Bildpunkte - von hinten lesen
   move.w #8-1, d4
   ploto:
    btst.b #0, d0               ;Bildpunkt hell oder dunkel?
    bne.s dunkelo               ;Bildpunkt ist dunkel
    hello:
    move.w #plotzoom-1, d5      ;Punkt in zoom-facher Laenge
    zoomo1:
     bsr vop                    ;1 Schritt vertikal nach oben
    dbra d5, zoomo1
    bra.s weitero
    dunkelo:
     move.w #plotzoom-1, d5     ;Punkt in zoom-facher Laenge
     zoomo2:
      bsr vo                    ;1 Schritt vertikal nach oben
     dbra d5, zoomo2
    weitero:
     ror.b #1, d0               ;naechster Bildpunkt
   dbra d4, ploto
  dbra d3, byteo
  move #400, zeit(A4)           ;Vorbereitung Rampe ungerade Zeile
  move.w #100, d4
  rampeo1:
   bsr vo                       ;1 Schritt vertikal nach oben
  dbra d4, rampeo1
  adda #64, A1                  ;Adresspointer wieder fr hin korrigieren
  zender:
  move #400, zeit(A4)           ;Zeilenabstand
  move.w #plotzoom, d4
  mulu #2, d4                   ;Es gibt nur 256 Zeilen, aber 512 Bildpunkte!
  zar:
   bsr hl                       ;1 Schritt vertikal nach links
  dbra d4, zar
 dbra d1, zeilen
 move.b #PENOFF, port           ;Stift in jedem Fall heben
rts

;* Fahren in Parkposition

park:
 bsr varinit                    ;Variablenintialisierung

 ;In Endlage vertikal unten fahren
 move #400, zeit(a4)            ;Zeit zwischen motorpulsen
 unten2:
  bsr vu                        ;1 Schritt vertikal unten
  move.b port, d5               ;Endlagenschalter auslesen
  btst.b #2, d5                 ;Endlagenschalter vertikal unten ausgelst?
  bne.s unten2
 ;Aus Endlage vertikal unten fahren ...
 move #400, zeit(a4)            ;Zeit zwischen Motorpulsen
 swvu1:
  bsr voe                       ;1 Schritt vertikal nach oben aus Endlage
  move.b port, d5               ;Endlagenschalter auslesen
  btst.b #2, d5                 ;Endlagenschalter vertikal unten frei?
  beq.s swvu1

 ;In Endlage horizontal rechts fahren
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  rechts2:
   bsr hr                       ;1 Schrittt horizontal rechts
   move.b port, d5              ;Endlagenschalter auslesen
   btst #0, d5                  ;Endlagenschalter horizontal rechts ausgeloest?
   bne.s rechts2
 ;Aus Endlage horizontal rechts fahren ...
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  swhr1:
   bsr hle                      ;1 Schritt horizontal nach links aus Endlage
   move.b port, d5              ;Endlagenschalter auslesen
   btst #0, d5                  ;Endlagenschalter horizontal rechts frei
  beq.s swhr1

 ;In die Parkposition fahren
  clr.l d0
  move #400, zeit(a4)           ;Zeit zwischen Motorpulsen
  oben2:
   bsr vo                       ;1 Schritt vertikal nach oben
   addq.l #1, d0
   cmp.l #200000, d0            ;Feste Strecke
  bne.s oben2
rts


;*** Schicht 3 - Variablendeklaration

varstart:

outbuff         equ 0                   ; 128 Byte
scalex          equ outbuff+128         ;   1 Langwort
scaley          equ scalex+4            ;   1 Langwort
vzx             equ scaley+4            ;   1 Byte
vzy             equ vzx+1               ;   1 Byte
faktor          equ vzy+1               ;   1 Wort
xwert           equ faktor+2            ;   1 Langwort
xrest           equ xwert+4             ;   1 Wort
ywert           equ xrest+2             ;   1 Langwort
yrest           equ ywert+4             ;   1 Wort
radius          equ yrest+2             ;   1 Langwort
zeit            equ radius+2            ;   1 Wort
dummy           equ zeit+2

varanz          equ *-varstart

end.
